import sys


def reload_class(instance):
    cls = instance.__class__
    modname = cls.__module__
    del sys.modules[modname]

    module = __import__(modname)
    submodules = modname.split(".")[1:]

    for submodule in submodules:
        module = getattr(module, submodule)

    instance.__class__ = getattr(module, cls.__name__)



#!/usr/bin/env python
import os
import re
import numpy as np

from pltrdy.rouge import read_rouge
from pltrdy.wc import wordcount


def srcrougefct(x):
    if x["src_rouge"] == "none":
        return 0.0
    else:
        return -float(x["src_rouge"].split(";")[0])


def perplexity(rouge_path, incl_oov=True):
    n_line = 0 if incl_oov else 1

    ppl_path = rouge_path.replace("rouge", "ppl")
    with open(ppl_path) as f:
        lines = [_.strip() for _ in f]
    line = lines[n_line]
    ppl = line.split("OOVs:")[-1]
    ppl = float(ppl)
    return ppl


def dual_xent(rouge_path, agg=np.mean, n=None):
    dual_xent_path = rouge_path.replace(".rouge", ".dual_xent.src-pred")

    xents = []
    with open(dual_xent_path) as f:
        for line in f:
            line = line.strip()
            xent = float(line)
            xents.append(xent)
    if n is not None:
        assert len(xents) == n

    return agg(xents)


def src_rouge(rouge_path):
    src_rouge_path = rouge_path.replace('.rouge', '.src_rouge')
    r = read_rouge(src_rouge_path)
    return r


def read_rouge_memory(rouge_path, memory={}, key="rouge"):
    if key in memory.keys():
        return memory[key]
    score = read_rouge(rouge_path)
    memory[key] = score
    return score


def step_field_fct(self, rouge_path, memory):
    name = os.path.basename(rouge_path)

    try:
        step = name.split("_pred.")[1].split("k")[0]
    except IndexError:
        raise IndexError("Cannot read step of '%s'" % rouge_path)
    return step


def dec_suffix_field_fct(self, rouge_path, memory):
    step = memory.get("step")
    if step is None:
        raise ValueError(
            "dec_suffix suppose 'step' field (absent from memory)")
    dec_suffix = rouge_path.split(str(step) + "k")[1]\
        .split(".txt")[0]\
        .replace('.', '')\
        .replace('true_test', '')
    return clean_suffix(dec_suffix)


def wc_field_fct(self, rouge_path, memory):
    wc = int(wordcount(rouge_path.replace('.rouge', '')))
    return wc


def src_rouge_field_fct(self, rouge_path, memory):
    r = src_rouge(rouge_path)
    return " ; ".join([
        "%2.2f" % (100 * float(r["rouge-1"][k]))
        for k in ["r", "p", "f"]
    ])


def model_name_by_pred(result_name):
    return result_name.split("_pred.")[0]


def clean_suffix(suffix):
    to_remove = ["bpe", "decoded", "rouge"]
    for r in to_remove:
        suffix = suffix.replace(r, "")
    return suffix


class ResultsExplorer(object):
    MODEL_NOTOK_REG = r'model_(.*)notok(.*)_pred(.*)'

    # Fields function
    # i.e. calculate field value from self, rouge_path, memory
    DEFAULT_FIELDS = {
        'exp': lambda s, p, m: os.path.dirname(p),
        'model': lambda s, p, m: s.model_from_result_name(os.path.basename(p)),
        'step': lambda s, p, m: step_field_fct(s, p, m),
        'dec_suffix': lambda s, p, m: dec_suffix_field_fct(s, p, m),
        'wc': lambda s, p, m: wc_field_fct(s, p, m),
        'rouge_1': lambda s, p, m: read_rouge_memory(p, m)["rouge-1"]["f"],
        'rouge_2': lambda s, p, m: read_rouge_memory(p, m)["rouge-2"]["f"],
        'rouge_l': lambda s, p, m: read_rouge_memory(p, m)["rouge-l"]["f"],
        'src_rouge': lambda s, p, m: src_rouge_field_fct(s, p, m)
    }

    FILTERS = {
        "valid": lambda p: ".valid" in p,
        "predtok": lambda p: ("predtok" in p or re.match(ResultsExplorer.MODEL_NOTOK_REG, p) is not None),
        "onlytoks": lambda p: "onlytoks" in p,
    }

    SORT_FCT = {
        "copy_desc": srcrougefct,
        "ppl": lambda x: -x["ppl"]
    }

    DEFAULT_REGEX = r"model(.*)\.rouge"

    def __init__(self, name, exps=[], default_regex=DEFAULT_REGEX, exps_with_regex={},
                 custom_filters={}, extra_fields={},
                 exclude_fields=[], fields=DEFAULT_FIELDS.keys(),
                 model_name_fct=model_name_by_pred):
        """
            name:
            exps: list of directories where *.rouge files are
            exps_with_regex: dict of `directory: regex`
            custom_filters: dict of filters `name: filter function`
            extra_fields: dict of `name: field function` that take a .rouge
                          path and should return the field value

        """
        self.name = name

        self.exps_with_regex = exps_with_regex
        self.exps_with_regex.update({
            e: default_regex
            for e in exps
        })

        self.fields = {}
        for k in fields:
            if k in exclude_fields:
                continue
            if k in ResultsExplorer.DEFAULT_FIELDS.keys():
                self.fields[k] = ResultsExplorer.DEFAULT_FIELDS[k]
            else:
                raise ValueError("Unknow field %s, choices are: %s"
                                 % (k, list(ResultsExplorer.DEFAULT_FIELDS[k])))
        self.fields.update(extra_fields)

        self.filters = dict(ResultsExplorer.FILTERS)
        self.filters.update(custom_filters)

        self.model_name_fct = model_name_fct

    def model_from_result_name(self, result_name):
        return self.model_name_fct(result_name)

    def all_filters(self, path, filters_switch):
        for k, v in filters_switch.items():
            _v = self.filters[k](path)
            print(path, k, v, _v)
            if not v == _v:
                return False
        return True
        return all([
            self.filters[k](path) == v
            for k, v in filters_switch.items()
        ])

    def explore_results(self, sort_field=None, **filters):
        fields = self.fields
        results = []

        for exp_root, reg in self.exps_with_regex.items():
            print(exp_root)
            exp_results = sorted([
                _ for _ in os.listdir(exp_root)
                if re.match(reg, _) is not None
                and self.all_filters(_, filters)
            ])
            for result_name in exp_results:
                rouge_path = os.path.join(exp_root, result_name)

                r = {}
                for key, fct in fields.items():
                    r[key] = fct(self, rouge_path, r)

                results.append(r)

        if sort_field is None:
            results = sorted(results,
                             key=lambda x: (
                                 x['rouge_1'],
                                 x['exp'],
                                 x['model'],
                                 x['step']
                             ),
                             reverse=True)
        else:
            if sort_field in ResultsExplorer.SORT_FCT:
                sort_fct = ResultsExplorer.SORT_FCT[sort_field]
            else:
                def sort_fct(x): return x[sort_field]
            results = sorted(results,
                             key=lambda x: (
                                 sort_fct(x),
                                 x['rouge_1'],
                                 x['exp'],
                                 x['model'],
                                 int(x['step'].replace('k', ''))
                             ),
                             reverse=True)

        assert len(results) > 0
        m = ["| " + "|  ".join(fields) + " |"]
        m += ["|---" * len(fields) + " |"]

        top = {k: max([r[k] for r in results])
               for k in ['rouge_1', 'rouge_2', 'rouge_l', 'wc']}
        for r in results:
            for k in ['rouge_1', 'rouge_2', 'rouge_l', 'wc']:
                if r[k] == top[k]:
                    r[k] = "**%f**" % r[k]

            _r = "| " + "| ".join([str(r[k]).replace('_', '_')
                                   for k in fields])
            _r += " |"
            m += [_r]

        filters_suffix = ""
        for k, v in filters.items():
            if v:
                filters_suffix += ".%s" % k

        sort_suffix = "" if sort_field is None else ".%s" % sort_field
        out_path = "results.%s%s%s.md" % (
            self.name, filters_suffix, sort_suffix)
        with open(out_path, 'w') as f:
            print("Output in '%s'" % out_path)
            print("\n".join(m), file=f)

    def cli(self, filters):
        import argparse

        parser = argparse.ArgumentParser()
        parser.add_argument("-sort_field", type=str)
        for f in filters:
            parser.add_argument("-%s" % f, action="store_true")

        args = parser.parse_args()
        filters_args = {
            f: getattr(args, f)
            for f in filters
        }
        self.explore_results(sort_field=args.sort_field, **filters_args)



import sys
import os

class Quiet:
    def __init__(self, out=None, err=None):
        if out is None:
            out = open(os.devnull, 'w')

        if err is None:
            err = open(os.devnull, 'w')

        self.stdout = None
        self.replace_out = out
        self.stderr = None
        self.replace_err = err

    def __enter__(self):
        self.stdout = sys.stdout
        sys.stdout = self.replace_out
        self.stderr = sys.stderr
        sys.stderr = self.replace_err
    
    def __exit__(self, type, value, traceback):
        sys.stdout = self.stdout
        sys.stderr = self.stderr





import pltrdy.percentiles
import pltrdy.assertion
import pltrdy.explore_results
import pltrdy.gen_script
import pltrdy.pickle
import pltrdy.rouge
import pltrdy.stats
import pltrdy.time
import pltrdy.tee
import pltrdy.wc  # noqa F401
import pltrdy.quiet
import pltrdy.tmpdir
import pltrdy.types

from pltrdy.describe import describe_str, describe_lines, describe, tab_print
from pltrdy.reload import reload_class

__version__ = "0.3.5"



import numpy as np


def equal(x, y, almost=False, epsilon=1e-7):
    if not almost:
        return x == y
    else:
        try:
            return (y - epsilon) <= x <= (y + epsilon)
        except TypeError:
            raise ValueError(
                "Invalid types (%s, %s) for almost comparison"
                % (type(x), type(y)))


def aeq(*args, almost=False, epsilon=1e-7, msg=None, ignore_single=False):
    if ignore_single and len(args) == 1:
        return args[0]

    assert len(args) > 1, (
        "aeq a single element is meaningless (ignore with `ignore_single`")

    if msg is not None:
        msg = "[%s] " % msg
    else:
        msg = ""
    assert all([
        equal(_,
              args[0],
              almost=almost,
              epsilon=epsilon)
        for _ in args[1:]
    ]), (
        "%sArguments are not all equal %s" % (msg, str(args))
    )
    return args[0]

def aaeq(*args, almost=False, epsilon=1e-7, msg=None, ignore_single=False):
    if ignore_single and len(args) == 1:
        return args[0]

    assert len(args) > 1, "aaeq a single element is meaningless, use aeq"
    if msg is not None:
        msg = "[%s] " % msg
    else:
        msg = ""

    aeq(*[len(_) for _ in args], msg="Size mismatch")
    for i, args_i in enumerate(zip(*args)):
        assert all([
            equal(_,
                  args_i[0],
                  almost=almost,
                  epsilon=epsilon)
            for _ in args_i[1:]]
        ), (
            "%sArguments are not all equal at position %d, %s."
            % (msg, i, str(args_i))
        )
    return args[0]


def assert_shapes(*shapes):
    """shapes are iterable of same length
       we assert that for all shape i and dimension j
       shapes[0][j] == shapes[i][j]
    """
    def fix_shape(shape):
        if isinstance(shape, np.ndarray):
            shape = shape.shape
        return shape
    shapes = [fix_shape(s) for s in shapes]

    for i, sizes in enumerate(zip(*shapes)):
        try:
            aeq(*sizes)
        except AssertionError as e:
            shapes_str = ", ".join([
                "[%s]" % (", ".join([
                    str(d) if j != i else "*%d*" % d
                    for j, d in enumerate(s)
                ])) for s in shapes
            ])
            raise AssertionError("Size mismatch on dimension %d: %s"
                                 % (i, shapes_str)) from e



import datetime
import pytz

EPOCH = "01/01/1970"
TZ_PARIS = "Europe/Paris"
DATE_FORMAT_DMY = "%d/%m/%Y"
DATE_FORMAT_ISO = "%Y-%m-%d %H:%M:%S"


def now(timezone=TZ_PARIS):
    timezone = pytz.timezone(timezone)
    now_tz = timezone.localize(datetime.datetime.now())
    return now_tz


def str_to_date(date_str, date_format=DATE_FORMAT_DMY, timezone=TZ_PARIS):
    timezone = pytz.timezone(timezone)
    date = datetime.datetime.strptime(date_str, date_format)
    datetz = timezone.localize(date)
    return datetz


def date_to_str(date, date_format=DATE_FORMAT_ISO):
    s = datetime.datetime.strftime(date, DATE_FORMAT_ISO)
    return s


def str_to_timestamp(date_str, date_format=DATE_FORMAT_DMY, timezone=TZ_PARIS):
    datetz = str_to_date(date_str, date_format, timezone)
    return datetz.timestamp()


def timestamp_to_str(t, date_format=DATE_FORMAT_ISO):
    d = datetime.datetime.fromtimestamp(t)
    s = date_to_str(date, date_format)
    return s



SEP_LINE = "---------------------------------------------"


def read_rouge(rouge_path):
    """Read ROUGE scores from ROUGE-1.5.5 output (or wrapper)
    Args:
        rouge_path(str): path of ROUGE file
    Returns:
        rouge_scores(dict): all scores,
            {$m: {$s: float}}
            for $m in ['rouge-1', 'rouge-2', 'rouge-l']
            for $m in ['f', 'r', 'p']
    """
    with open(rouge_path) as f:
        lines = [_.strip() for _ in f]

    rouge = {}
    for i, line in enumerate(lines):
        if line == SEP_LINE:
            # print("Ignore SEP line")
            continue
        elif line.startswith("1 "):
            line = line[2:]

            metric, stat_and_score = line.split("Average")

            assert metric.startswith("ROUGE-")
            metric = metric.lower().strip()

            stat, score = stat_and_score.split(":")
            assert stat.startswith("_")
            stat = stat.replace("_", "").lower()

            score_value, confidence = score.split("(")
            score_value = float(score_value)

            conf_prct, conf_int = confidence.split("-conf.int. ")
            assert conf_prct.endswith("%")
            conf_prct = float(conf_prct.replace("%", "")) / 100

            assert conf_int.endswith(")")
            conf_lower, conf_upper = conf_int.replace(")", "").split(" - ")

            if not metric in rouge.keys():
                rouge[metric] = {}
            if stat in rouge[metric].keys():
                raise ValueError("Mutliple value for %s %s at line %d"
                                 % (metric, stat, i))
            rouge[metric][stat] = score_value

        elif len(line) == 0:
            # print("Ignore empty line")
            continue
        else:
            raise ValueError("Unkown line (%d): %s" % (i, line))

    return rouge





def checktype(o, *types):
    for i, t in enumerate(types):
        if isinstance(t, type):
            if isinstance(o, t):
                break

        elif t is None:
            if o is None:
                break

        else:
            raise TypeError(
                f"Invalid type of type {i} during check: {type(t)} instead of "
                f"type, None")
    else:
        raise TypeError(
            f"Invalid type {type(o)}, must be {', '.join(map(str, types))}")



import pickle


def load(*args, **kwargs):
    return pickle_load(*args, **kwargs)


def save(*args, **kwargs):
    return pickle_save(*args, **kwargs)


def pickle_load(path):
    with open(path, 'rb') as f:
        return pickle.load(f)


def pickle_save(o, path):
    with open(path, 'wb') as f:
        pickle.dump(o, f)



#!/usr/bin/env python
import json


def test():
    from gen_script import gen_script

    data = {
        "name": "the_test",
        "args": ["param1", "p2", ],
        "kwargs": {"opt_param1": "value1", "opt_param2": "value2"}
    }

    data_json = json.dumps(data)
    gen_script(data_json)


if __name__ == "__main__":
    test()



import tempfile


class TemporaryDirectory(tempfile.TemporaryDirectory):
    """A TemporaryDirectory similar to tempfile but which can be detached if
       `keep` is True i.e. the directory isn't cleaned up on object garbage
        collection
    """
    def __init__(self, *args, keep=False, **kwargs):
        super(TemporaryDirectory, self).__init__(*args, **kwargs)

        self.keep = keep
        if self.keep:
            self._finalizer.detach()

    def __exit__(self, exc, value, tb):
        if not self.keep or exc is None:
            self.cleanup()
        else:
            _warnings.warn(f"Not cleaning up {self.name} due to exception. "
                           f"Set `keep=False` to force cleanup",
                           ResourceWarning)




# tee.py
# Purpose: A Python class with a write() method which, when
# used instead of print() or sys.stdout.write(), for writing
# output, will cause output to go to both sys.stdout and
# the filename passed to the class's constructor. The output
# file is called the teefile in the below comments and code.

# The idea is to do something roughly like the Unix tee command,
# but from within Python code, using this class in your program.

# The teefile will be overwritten if it exists.

# The class also has a writeln() method which is a convenience
# method that adds a newline at the end of each string it writes,
# so that the user does not have to.

# Python's string formatting language is supported (without any
# effort needed in this class), since Python's strings support it,
# not the print method.

# Author: Vasudev Ram
# Web site: https://vasudevram.github.io
# Blog: https://jugad2.blogspot.com
# Product store: https://gumroad.com/vasudevram

from __future__ import print_function
import sys


class Tee(object):
    RAISE = "raise"
    EXIT = "exit"

    def __init__(self, tee_filename, behavior=RAISE):
        self.behavior = behavior.lower()
        assert self.behavior in [Tee.RAISE, Tee.EXIT, ]

        try:
            self.tee_fil = open(tee_filename, "w")
        except IOError as ioe:
            if self.behavior == Tee.RAISE:
                raise
            else:
                exit("Caught IOError: {}".format(repr(ioe)))
        except Exception as e:
            if self.behavior == Tee.RAISE:
                raise
            else:
                exit("Caught Exception: {}".format(repr(e)))

    def write(self, s):
        sys.stdout.write(s)
        self.tee_fil.write(s)

    def writeln(self, s):
        self.write(s + '\n')

    def close(self):
        try:
            self.tee_fil.close()
        except IOError as ioe:
            if self.behavior == Tee.RAISE:
                raise
            else:
                exit("Caught IOError: {}".format(repr(ioe)))
        except Exception as e:
            if self.behavior == Tee.RAISE:
                raise
            else:
                exit("Caught Exception: {}".format(repr(e)))

    def __enter__(self):
        return self.tee_fil

    def __exit__(self, exc_type, exc_val, exc_tb):
        self.close()



#!/usr/bin/env python
import sys
import json


def gen_script(data):
    if isinstance(data, str):
        try:
            data = json.loads(data)
        except BaseException:
            print("data: '%s'" % str(data))
            raise
    tab = "    "
    python_bin = "/usr/bin/env python"
    fct_name = data["name"]
    imports = data.get("import", [])
    args = data.get("args", [])
    kwargs = data.get("kwargs", {})
    all_args = args + ['%s="%s"' % (k, v) for k, v in kwargs.items()]
    all_args_str = ", ".join(all_args)

    def tprint(tlvl, *args, **kwargs):
        args = list(args)
        args[0] = "%s%s" % (tlvl * tab, args[0])
        print(*args, **kwargs)

    print("#!%s" % python_bin)
    for module in imports:
        print("import %s" % module)
    print("\n")
    print("def %s(%s):" % (fct_name, all_args_str))
    tprint(1, "pass\n\n")
    print('if __name__ == "__main__":')
    tprint(1, "import argparse\n")
    tprint(1, "parser = argparse.ArgumentParser()")

    main_args = []
    main_kwargs = []
    for arg in args:
        tprint(1, 'parser.add_argument("%s",' % arg)
        tprint(1, '                    help="")')
        main_args.append("args.%s" % arg)
    for k, v in kwargs.items():
        tprint(
            1, 'parser.add_argument("-%s", "--%s", default="%s",' %
            (k, k, v))
        tprint(1, '                    help="")')
        main_kwargs.append("%s=args.%s" % (k, k))

    main_args_str = ", ".join(main_args + main_kwargs)
    tprint(1, "args = parser.parse_args()\n")
    tprint(1, "%s(%s)" % (fct_name, main_args_str))


def main():
    data = None
    if len(sys.argv) >= 1:
        data = sys.argv[1]

    gen_script(data)


if __name__ == "__main__":
    main()



import sys


def binsize(o):
    import dill
    return len(dill.dumps(o))


def has_module(module_name):
    return module_name in sys.modules


def is_torch_tensor(o):
    if has_module("torch"):
        import torch

        if isinstance(o, torch.Tensor):
            return True
    return False


def tab_print(*args, sep="        ", lvl=0, **kwargs):
    args = list(args)
    if lvl > 0:
        tabs = sep * lvl
        args[0] = tabs + args[0]
    print(*args, **kwargs)


def describe(o, max_elements=20, max_depth=100, depth=1, file=sys.stdout,
             report_size=False):
    def _print(*args, **kwargs):
        print(*args, **kwargs, file=file)

    next_kwargs = {
        "max_elements": max_elements,
        "max_depth": (max_depth - 1),
        "depth": (depth + 1),
        "file": file,
        "report_size": report_size,
    }

    lines = []

    if report_size:
        size = binsize(o)
        size_str = ", binsize: %d o" % size
    else:
        size_str = ""

    if max_depth == 0:
        _print("(max depth reached)")
        return

    if isinstance(o, dict):
        keys = o.keys()
        n = len(o)
        _print("Dict (len: %d%s)" % (n, size_str))
        if n <= max_elements:
            try:
                keys = sorted(o.keys())
            except TypeError:
                keys = sorted(o.keys(), key=lambda e: str(e))

            for k in keys:
                v = o[k]
                tab_print("%s:" % k, lvl=depth, end=" ", file=file)
                describe(v, **next_kwargs)
        else:
            tab_print("(too many elements to show, %d > %d)"
                      % (n, max_elements), lvl=depth, file=file)
    elif isinstance(o, list) or isinstance(o, tuple):

        n = len(o)
        _print("%s (len: %d%s)" % (type(o), n, size_str))
        if n <= max_elements:
            for i, v in enumerate(o):
                tab_print("#%d:" % i, lvl=depth, end=" ", file=file)
                describe(v, **next_kwargs)
        else:
            tab_print("(too many elements to show, %d > %d)"
                      % (n, max_elements), lvl=depth, file=file)
    elif is_torch_tensor(o):
        tensor_shape = str(list(o.size()))
        tensor_type = str(o.type())

        _print("%s: %s%s" % (tensor_type, tensor_shape, size_str))
    else:
        _print("%s%s" % (repr(o), size_str))


def describe_str(*args, **kwargs):
    from io import StringIO
    s = StringIO()
    describe(*args, file=s, **kwargs)

    return s.getvalue()


def describe_lines(*args, **kwargs):
    return describe_str(*args, **kwargs).split("\n")



#!/usr/bin/env python
import os
import re

from setuptools import setup, find_packages


this_directory = os.path.abspath(os.path.dirname(__file__))
with open(os.path.join(this_directory, 'README.md'), encoding='utf-8') as f:
    long_description = f.read()


def get_version():
    path = os.path.join(os.path.dirname(__file__), 'pltrdy', '__init__.py')
    with open(path, 'r') as f:
        content = f.read()
    m = re.search(r'__version__\s*=\s*"(.+)"', content)
    assert m is not None
    return m.group(1)


setup(
    name='pltrdy',
    description='',
    long_description=long_description,
    long_description_content_type='text/markdown',
    version=get_version(),
    packages=find_packages(),
    project_urls={
    },
    install_requires=[
        "pytz",
    ],
    entry_points={
        "console_scripts": [
            "gen_py=pltrdy.gen_script:main"
        ],
    }
)



import numpy as np


def percentiles(data, n):
    """Return all (n-1) values of n-ciles over the data
       i.e. if n=10, it returns deciles therefore 9 values.
       i.e. percentiles(data, n)[int((n/2)-1)] = median(percentiles)

    """
    prct = 100 / n
    cur = prct
    q = []
    while cur < 100:
        q.append(cur)
        cur += prct

    p = list(np.percentile(data, q))
    assert len(p) == (n - 1)
    return p


def deciles(data):
    return percentiles(data, n=10)


def quartiles(data):
    return percentiles(data, n=4)



#!/usr/bin/env python
"""
    Python wrapper around `wc` command
"""
import subprocess


def linecount(path):
    return wordcount(path, args="-l")


def wordcount(path, args=["-w"]):
    o = subprocess.run(["wc"] + args + [path], stdout=subprocess.PIPE)
    wc = int(o.stdout.decode('utf-8').split()[0])
    return wc
